#!/etc/opt/openit/perl

###############################################################################
#
# Excludes sentinel products from calculating feature version 'all' and use
# feature version 'smt' as 'all' instead.
#
#  - licpoll2 filters
#

BEGIN {
    push @INC, "/opt/openit/contrib/lib";
}

use strict;

use OpeniT::Contrib::Config;

my $config = new OpeniT::Contrib::Config;               # Reads default config file(s)
$config->read("/etc/opt/openit/openit_contrib.cfg");    # Additional config

#
# Get Open iT config directory (and verify value)
#
my $cfg_dir = $config->{CONFIG_DIR};
if (! -d $cfg_dir) {
    die "Unable to locate Open iT config ($cfg_dir) directory";
}

my $file_name = "sentinel-products.map";
my $map_file = "${cfg_dir}/$file_name";
my $filter_var = "var4";

# Get list of filters to update

# get all licpoll2 
my @filter_files = &get_filters_to_update("$cfg_dir/filtrator");

foreach my $filter (@filter_files) {
    &update_filter( $filter );
}
&create_mapping_file($map_file);

exit 0;


###############################################################################
#
# Scan filter directory and get list of files to update
#
sub get_filters_to_update {
    my ($filter_dir) = @_;
    
    my @filters = glob("$filter_dir/*.conf");
    my @licpoll2_filters;
    
    foreach my $filter (@filters) {
	open(FILE, $filter) || die "Unable to read filter file '$filter'";
	my @lines = <FILE>;
	close(FILE);

	if( grep {/type\(String\)=\"licpoll2\"/} @lines ) {
		push( @licpoll2_filters, $filter );
	}
    }

    return @licpoll2_filters;
}


###############################################################################
#
# Update filter file with new filter
#
sub update_filter {
    my ($filter) = @_;

    print "Reading '$filter'\n";

    open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;
    close(FILE);

    # Check that filter update is not already present
    if ( grep {/Mapping:sentinel-products|Mapping:smt-to-all/} @lines ) {
        print "Filter '$filter' already updated\n";
        return;
    }

    if( !(grep(/Duplicate:license-usage/, @lines) || grep(/Duplicate:version/, @lines)) ) 
    {
	print "No need to update filter. No Duplicate:license-usage or Duplicate:version is found.\n";
	return;
    }

    #
    # Locate position to insert Mapping call
    #

    # Scan to start of filter-process and add "call" as first
    # object
    my $ind = 0;
    $ind++ while( $lines[$ind] !~ /filter-process/ );
    $ind += 2;
    my @insert = ("\t\tMapping:sentinel-products()=\"\"\n\t\t{\n",
		  "\t\tMapping:smt-to-all()=\"\"\n\t\t{\n");

    splice @lines, $ind, 0, @insert;
    $ind += 2; # Skip inserted lines

    # Now find filter def and insert closing '}' and then new filter
    $ind++ while( $lines[$ind] !~ /filter-defs/ );
    splice @lines, $ind-2, 0, "\t\t}\n\t\t}\n";
    $ind += 3; # Compensate for inserted line and then into object
    @insert = ("\t\tMapping:sentinel-products()=\"\"\n",
	       "\t\t{\n",
	       "\t\t\tinactive(Bool)=\"false\"{}\n",
	       "\t\t\tfilter(String)=\"Mapping\"{}\n",
	       "\t\t\tmap-file(FileName)=\"\${OpeniT.directories.conf}$file_name\"{}\n",
	       "\t\t\tsource(String)=\"product\"{}\n",
	       "\t\t\ttarget(String)=\"$filter_var\"{}\n",
	       "\t\t}\n",
	       "\t\tMapping:smt-to-all()=\"\"\n",
	       "\t\t{\n",
	       "\t\t\tinactive(Bool)=\"false\"{}\n",
	       "\t\t\tfilter(String)=\"Mapping\"{}\n",
	       "\t\t\tlimiting(String)=\"\$$filter_var == 'SENTINEL_PRODUCT'\"{}\n",
	       "\t\t\tsimple-map(String)=\"'smt'='all'\"{}\n",
	       "\t\t\tsource(String)=\"feature_version\"{}\n",
	       "\t\t\ttarget(String)=\"feature_version\"{}\n",
	       "\t\t}\n" );

    splice @lines, $ind, 0, @insert;

    my $line_count = @lines;

    # Next is find Duplicate:license-usage and add a limiting condition that excludes Sentinel products
    $ind++ while( $lines[$ind] !~ /Duplicate:license-usage/ && $lines[$ind] !~ /Duplicate:version/ );
    $ind++ while( $lines[$ind] !~ /filter\(String\)/ );
    $ind++;

    @insert = ("\t\t\tlimiting(String)=\"\$$filter_var != 'SENTINEL_PRODUCT'\"{}\n");    
    splice @lines, $ind, 0, @insert;

    my $new = "$filter.new";
    my $old = "$filter.old";

    # Create new tmp file
    open(OUT, ">$new") || die "Unable to create tmp file '$new'";
    print OUT @lines;
    close(OUT);

    # Mv old file to .old
    if ( ! rename $filter, $old ) {
        die "Unable to rename '$filter'";
    }

    # Rename new
    if ( ! rename $new, $filter ) {
        die "Unable to rename '$new'";
    }

}



###############################################################################
#
# Add mapping file
#
sub create_mapping_file {
    my ($map) = @_;

    # Check that update has not already been done

    if(-e $map) {
	print "Mapping file allready present\n";
	return;
    }

    print "Adding mapping file '$map'\n";

    open(OUT, ">$map") || die "Unable to open map file '$map' for writing";
    print OUT "# This mapping file includes the products which use Sentinel LM.
# There will be a special behavior to calculate feature version 'all' on the products
# included here. Please add the sentinel products you have.

MAP: SENTINEL_PRODUCT

product1
product2
add-products-here
";

    close(OUT);
}
